<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\StoreAssetRequest;
use App\Models\Asset;
use App\Models\City;
use App\Models\Country;
use App\Models\Entity;
use App\Models\Region;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;

class AssetController extends Controller
{
    public function assetDescriptions()
    {
        $assets = Asset::all();

        if (!$assets->isEmpty()) {

            $descriptions = []; // Store unique descriptions
            $data = [];
            foreach ($assets as $asset) {
                // Check if description already exists in $descriptions array
                if (!in_array($asset->asset_description, $descriptions)) {
                    // If not, add it to $descriptions and $data array
                    $descriptions[] = $asset->asset_description;
                    $data[] = [
                        'id' => $asset->id,
                        'name' => $asset->asset_description
                    ];
                }
            }
            return response()->json([
                'message' => 'Asset descriptions found successfully!',
                'data' => [
                    'payload' =>  $data
                ],
                'status' => 200
            ]);
        } else {
            return response()->json([
                'message' => 'Assets not found!',
                'status' => 404
            ]);
        }
    }

    public function storeAssets(Request $req)
    {
        $user = Auth::guard('sanctum')->user();

        $validator = Validator::make($req->all(), [
            "asset_tag" => "required",
            "company_id" => "required",
            "quantity" => "required",
            "asset_description" => "required",
            "lat" => "required",
            "lng" => "required",
            "room_number" => "required",
            "floors_number" => "required",
            "building_number" => "required",
            "city" => "required",
            "region" => "required",
            "country" => "required",
            "asset_type" => "required",
            "manufacturer" => "required",
            "asset_have_serial_number" => "nullable",
            "supportive_document_for_acquisition_cost" => "nullable",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => [
                "message" => $validator->errors()->first(),
                'status' => 422
            ]], 200);
        }

        $input = $req->all();
        $tags = explode(',', $req->asset_tag);

        $tagsWithErrors = [];

        foreach ($tags as $tag) {
            $validator = Validator::make(['asset_tag' => $tag], [
                'asset_tag' => 'required|unique:assets,tag_number',
            ]);

            if ($validator->fails()) {
                $tagsWithErrors[] = $validator->errors()->first() . '( ' . $tag . ' )';
            }
        }

        if (!empty($tagsWithErrors)) {
            return response()->json([
                'errors' => [
                    'message' => $tagsWithErrors,
                    'status' => 422
                ]
            ], 200);
        }

        $entity = Entity::where('entity_code', $req->company_id)->first();

        if (!$entity) {
            return response()->json([
                'message' => 'Entity Not Found!',
                'status' => 404
            ]);
        }

        $country = Country::findOrFail($input['country']);
        $region = Region::findOrFail($input['region']);
        $city = is_numeric($input['city']) ? City::findOrFail($input['city']) : City::firstOrCreate(['name' => $input['city'], 'region_id' => $input['region']]);

        $paths = [];
        if ($req->hasFile('asset_images')) {
            foreach ($req->file('asset_images') as $file) {
                $path = $file->store('asset_images');
                if ($path) {
                    $paths[] = $path;
                } else {
                    return response()->json([
                        'message' => 'Failed to store asset images. Please check your internet connection.',
                        'status' => 500
                    ]);
                }
            }
        } else {
            return response()->json([
                'message' => 'No asset images provided.',
                'status' => 400
            ]);
        }

        try {
            DB::beginTransaction();

            foreach ($tags as $tag) {
                $asset = Asset::create([
                    'asset_owner' => $user->username,
                    'asset_type' => $input['asset_type'],
                    'entity_id' => $entity->id,
                    'asset_description' => $input['asset_description'],
                    'country' => $country->name,
                    'region' => $region->name,
                    'city' => $city->name,
                    'building_number' => $input['building_number'],
                    'floor_number' => $input['floors_number'],
                    'room_office_number' => $input['room_number'],
                    'quantity' => $input['quantity'],
                    'longitute' => $input['lng'],
                    'latitude' => $input['lat'],
                    'tag_number' => $tag,
                    'admin_status' => 'Pending for data entry',
                    'manufacturer' => $input['manufacturer'],
                    'asset_have_serial_number' => $input['asset_have_serial_number'],
                    'supportive_document_for_acquisition_cost' => $input['supportive_document_for_acquisition_cost'] ?? null,
                ]);

                if (!$asset) {
                    DB::rollBack();
                    return response()->json([
                        'message' => 'Failed to create assets. Please check your internet connection.',
                        'status' => 500
                    ]);
                }

                $asset->attachments()->createMany(
                    collect($paths)->map(function ($path) {
                        return ['url' => $path];
                    })->all()
                );
            }

            DB::commit();
            return response()->json([
                'message' => 'Assets created successfully!',
                'status' => 200
            ]);
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Asset creation failed', $e->getMessage());

            return response()->json([
                'message' => 'Failed to store asset images. Please check your internet connection.',
                'status' => 500
            ]);
        }
    }


    public function buildingList()
    {
        $assets = Asset::all();

        if (!$assets->isEmpty()) {

            $buildingNumbers = []; // Store unique building numbers
            $data = [];
            foreach ($assets as $asset) {
                // Check if building number already exists in $buildingNumbers array
                if (!in_array($asset->building_number, $buildingNumbers)) {
                    // If not, add it to $buildingNumbers and $data array
                    $buildingNumbers[] = $asset->building_number;
                    $data[] = [
                        'id' => $asset->id,
                        'name' => $asset->building_number
                    ];
                }
            }
            return response()->json([
                'message' => 'Asset Building Numbers found successfully!',
                'data' => [
                    'payload' =>  $data
                ],
                'status' => 200
            ]);
        } else {
            return response()->json([
                'message' => 'Assets not found!',
                'status' => 404
            ]);
        }
    }


    public function floorList()
    {
        $assets = Asset::all();

        if (!$assets->isEmpty()) {

            $floorNumbers = []; // Store unique floor numbers
            $data = [];
            foreach ($assets as $asset) {
                // Check if floor number already exists in $floorNumbers array
                if (!in_array($asset->floor_number, $floorNumbers)) {
                    // If not, add it to $floorNumbers and $data array
                    $floorNumbers[] = $asset->floor_number;
                    $data[] = [
                        'id' => $asset->id,
                        'name' => $asset->floor_number
                    ];
                }
            }
            return response()->json([
                'message' => 'Asset Floor Numbers found successfully!',
                'data' => [
                    'payload' =>  $data
                ],
                'status' => 200
            ]);
        } else {
            return response()->json([
                'message' => 'Assets not found!',
                'status' => 404
            ]);
        }
    }


    public function roomList()
    {
        $assets = Asset::all();

        if (!$assets->isEmpty()) {

            $roomNumbers = []; // Store unique room/office numbers
            $data = [];
            foreach ($assets as $asset) {
                // Check if room/office number already exists in $roomNumbers array
                if (!in_array($asset->room_office_number, $roomNumbers)) {
                    // If not, add it to $roomNumbers and $data array
                    $roomNumbers[] = $asset->room_office_number;
                    $data[] = [
                        'id' => $asset->id,
                        'name' => $asset->room_office_number
                    ];
                }
            }
            return response()->json([
                'message' => 'Asset Room/Office Numbers found successfully!',
                'data' => [
                    'payload' =>  $data
                ],
                'status' => 200
            ]);
        } else {
            return response()->json([
                'message' => 'Assets not found!',
                'status' => 404
            ]);
        }
    }

    public function barcodeList()
    {
        $assets = Asset::pluck('tag_number')->all();

        if (!empty($assets)) {
            return response()->json([
                'message' => 'Asset Barcodes found successfully!',
                'barcode_list' => $assets,
                'status' => 200
            ]);
        } else {
            return response()->json([
                'message' => 'Assets not found!',
                'status' => 404
            ]);
        }
    }
}
