<?php

namespace App\Http\Controllers;

use App\Models\Asset;
use App\Models\Department;
use Illuminate\Http\Request;
use Illuminate\Database\Eloquent\Collection;
use Yajra\DataTables\DataTables;

class DepartmentController extends Controller
{
    public function index(Request $req)
    {
        if (!$this->authorize('list-department')) {
            abort(403);
        }

        if ($req->ajax()) {

            $departments = Department::with('childrenRecursive')->whereNull('departments_id')->get();
            $table_data = [];

            foreach ($departments as $department) {
                $this->addDepartmentHierarchy($table_data, $department);
            }

            $collection = new Collection($table_data);
            return DataTables::of($collection)
                ->addIndexColumn()
                ->addColumn('action', function ($query) {
                    return view('admin.partials.action', [
                        'edit_route' => route('edit.department', ['department' => $query['id']]),
                        'delete_route' => route('delete.department', ['department' => $query['id']]),
                        'edit_permission' => 'edit-department',
                        'delete_permission' => 'delete-department',
                    ]);
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view('admin.department.index');
    }

    private function addDepartmentHierarchy(&$table_data, $department, $parent_name = '', $full_name = '')
    {
        $full_name = $full_name ? $full_name . ' / ' . $department->name : $department->name;

        $table_data[] = [
            'id' => $department->id,
            'name' => $department->name,
            'parent_name' => $parent_name,
            'full_name' => $full_name
        ];

        foreach ($department->children as $child) {
            $this->addDepartmentHierarchy($table_data, $child, $department->name, $full_name);
        }
    }



    public function edit(department $department)
    {
        if (!$this->authorize('edit-department')) {
            abort(403);
        }

        $departments = Department::where('id', '!=', $department->id)->get();

        return view('admin.department.edit', ['department' => $department, 'departments' => $departments]);
    }

    public function update(Request $req, department $department)
    {
        if (!$this->authorize('edit-department')) {
            abort(403);
        }

        $department->name = $req->name;
        $department->departments_id = $req->departments_id;
        $department->save();

        session()->forget('departments');
        session()->forget('departments_expiry');

        toast('Department updated', 'success');
        return redirect()->route('departments');
    }

    public function delete(department $department)
    {
        if (!$this->authorize('delete-department')) {
            abort(403);
        }

        $asset = Asset::where('department_id', $department->id)->first();

        if ($asset) {
            $asset->department_id = null;
            $asset->save();
        }

        $department->delete();

        session()->forget('departments');
        session()->forget('departments_expiry');

        toast('department Deleted!', 'success');
        return redirect()->back();
    }

    public function create()
    {
        if (!$this->authorize('create-department')) {
            abort(403);
        }
        $departments = Department::all();
        return view('admin.department.form', ['departments' => $departments]);
    }

    public function store(department $department, Request $req)
    {
        if (!$this->authorize('create-department')) {
            abort(403);
        }
        $department->name = $req->name;
        $department->departments_id = $req->departments_id;
        $department->save();

        session()->forget('departments');
        session()->forget('departments_expiry');

        toast('department Created Successfully!', 'success');
        return redirect()->route('departments');
    }
}
