<?php

namespace App\Http\Controllers;

use App\Jobs\AssetImageUnclearJob;
use App\Models\Asset;
use App\Models\City;
use App\Models\Country;
use App\Models\Level1;
use App\Models\Level2;
use App\Models\Level3;
use App\Models\Level4;
use App\Models\Region;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Database\Eloquent\Collection;
use Yajra\DataTables\Facades\DataTables;

class FlagAssetController extends Controller
{
    public function index(Request $req)
    {
        if (!$this->authorize('list-flag_asset')) {
            abort(403);
        }

        $descriptions = Asset::distinct()->pluck('asset_description');
        $countries = Country::all();
        $regions = Region::all();
        $cities = City::all();
        $level3s = Level3::all();
        $buildings = Asset::distinct()->pluck('building_number');
        $floors = Asset::distinct()->pluck('floor_number');
        $rooms = Asset::distinct()->pluck('room_office_number');
        $data_entry_users = User::role(User::DATA_ENTRY_ROLE)->get();
        $field_users = User::role(User::FIELD_USER_ROLE)->get();

        return view('admin.asset.flag_asset', compact('descriptions', 'countries', 'regions', 'cities', 'level3s', 'buildings', 'floors', 'rooms', 'data_entry_users', 'field_users'));
    }

    public function clear(Request $req)
    {
        $asset = Asset::where('id', $req->id)->update(['flag' => "Clear"]);
        toast('Flag Updated Successfully', 'success');
        return redirect()->back();
    }

    public function reFlag(Request $req)
    {

        $asset = Asset::where('id', $req->id)->first();
        $assetUpdate = Asset::where('id', $req->id)->update(['flag' => "Not Clear"]);
        $user = User::where('username', $asset->asset_owner)->first();

        if ($user) {
            $mailData = [
                'title' => 'Mail From Qiam',
                'body' => "Hello $user->username! Your Asset Images is un clear please check it. Thank you.",
                'email' => $user->email,
                'id' => $req->id,
            ];

            AssetImageUnclearJob::dispatch($mailData);
        }

        toast('Flag Updated Successfully', 'success');
        return redirect()->back();
    }

    public function getFlagAssets(Request $req)
    {
        $draw = $req->get('draw'); // Internal use
        $start = $req->get("start"); // where to start next records for pagination
        $rowPerPage = $req->get("length"); // How many records needed per page for pagination

        $assets = Asset::query()->where('flag', "Not Clear");

        if (Auth::user()->hasRole(User::SUPER_ADMIN_ROLE)) {
            $assets->orderBy('created_at', 'desc');
        } elseif (Auth::user()->hasRole(User::SUPERVISOR_ROLE) && Auth::user()->hasRole(User::DATA_ENTRY_ROLE)) {
            $assets->where(function ($query) {
                $query->where('region', Auth::user()->region)
                    ->orWhere('asset_assign', Auth::user()->username);
            })->orderBy('created_at', 'desc');
        } elseif (Auth::user()->hasRole(User::SUPERVISOR_ROLE)) {
            $assets->where('region', Auth::user()->region);
        } elseif (Auth::user()->hasRole(User::DATA_ENTRY_ROLE)) {
            $assets->where('asset_assign', Auth::user()->username);
        } elseif (Auth::user()->hasRole(User::FIELD_USER_ROLE)) {
            $assets->where('asset_owner', Auth::user()->username);
        }

        // Filter by asset type if provided
        if ($req->has('asset_type')) {
            // dd($req->asset_type);
            $assets->where('asset_type', 'like', '%' . $req->asset_type . '%');
        }
        if ($req->has('description')) {
            $assets->where('asset_description', 'like', '%' . $req->description . '%');
        }
        if ($req->has('country')) {
            $assets->where('country', 'like', '%' . $req->country . '%');
        }
        if ($req->has('region')) {
            $assets->where('region', 'like', '%' . $req->region . '%');
        }
        if ($req->has('city')) {
            $assets->where('city', 'like', '%' . $req->city . '%');
        }
        if ($req->has('barcode_number')) {
            $assets->where('tag_number', 'like', '%' . $req->barcode_number . '%');
        }
        if ($req->has('building_number')) {
            $assets->where('building_number', 'like', '%' . $req->building_number . '%');
        }
        if ($req->has('floor_number')) {
            $assets->where('floor_number', 'like', '%' . $req->floor_number . '%');
        }
        if ($req->has('room_office_number')) {
            $assets->where('room_office_number', 'like', '%' . $req->room_office_number . '%');
        }
        if ($req->has('level_3')) {
            if ($req->level_3) {
                $data = Level3::where('name', $req->level_3)->first();
                if ($data) {
                    $assets->where('level3', $data->id);
                }
            }
        }
        if ($req->has('multi_tags')) {
            if ($req->multi_tags) {

                $tags = explode(',', $req->multi_tags);

                $assets->whereIn('tag_number', $tags);
            }
        }

        if ($req->has('asset_assign')) {
            if ($req->asset_assign) {
                $assets->where('asset_assign', $req->asset_assign);
            }
        }
        if ($req->has('asset_owner')) {
            if ($req->asset_owner) {
                $assets->where('asset_owner', $req->asset_owner);
            }
        }

        // Execute the query
        $total = $assets->count();
        $totalFilter = $total;

        $arrData = $assets->skip($start)->take($rowPerPage)->get();


        // Modify the data structure to include HTML for the checkbox column
        $data = [];

        foreach ($arrData as $asset) {
            $l3 = Level3::where('id', $asset->level3)->value('name_ar');
            $assetType = $asset->asset_type === 'plantAndMachinery' ? 'Plant & Machinery' : $asset->asset_type;

            $buttons = view('admin.partials.action', [
                'edit_route' => route('edit.asset', ['asset' => $asset['id'], 'type' => $asset['asset_type']]),
                'view_route' => route('show.asset', ['asset' => $asset['id']]),
                'export_route' => route('export.asset', ['asset' => $asset['id']]),
                'delete_route' => route('delete.asset', ['asset' => $asset['id']]),
                'assign_route' => route('assign.asset', ['asset' => $asset['id']]),
                'image_download_route' => route('download.image', ['asset' => $asset['id']]),
                'image_download_permission' => 'download-asset-images',
                'edit_permission' => 'edit-asset',
                'delete_permission' => 'delete-asset',
                'view_permission' => 'view-asset',
                'export_permission' => 'export-asset',
                'assign_permission' => 'assign-asset',
            ])->render();

            $data[] = [
                'id' => $asset->id,
                'asset_type' => $assetType,
                'tag_number' => $asset->tag_number,
                'level3' => $l3,
                'country' => $asset->country,
                'region' => $asset->region,
                'city' => $asset->city,
                'flag' => $asset->flag,
                'user_status' => $asset->user_status,
                'admin_status' => $asset->admin_status,
                'building_number' => $asset->building_number,
                'floor_number' => $asset->floor_number,
                'room_office_number' => $asset->room_office_number,
                'asset_assign' => $asset->asset_assign,
                'asset_description' => $asset->asset_description,
                'image' => $asset->attachments->first()
                    ? asset('storage/' . $asset->attachments->first()->url)
                    : asset('admin/img/add-photo2.png'),
                'allImage' => $asset->attachments
                    ? $asset->attachments->pluck('url')->toArray()
                    : asset('admin/img/add-photo2.png'),
                'action' => $buttons,
            ];
        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $total,
            "recordsFiltered" => $totalFilter,
            "data" => $data,
        );

        return response()->json($response);
    }
}
