<?php

namespace App\Http\Controllers;

use App\Http\Requests\RoleStoreRequest;
use App\Models\Permission;
use App\Models\Role;
use App\Models\User;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Artisan;

class RoleController extends Controller
{
    public function index(Request $req)
    {
        if (!$this->authorize('list-role')) {
            abort(403);
        }
        if ($req->ajax()) {
            // Get All Roles Without Super-Admin
            $roles = Role::where('name', '!=', User::SUPER_ADMIN_ROLE)->get();
            $table_data = [];
            foreach ($roles as $data) {
                $table_data[] = [
                    'id' => $data->id,
                    'name' => $data->name,
                    'permissions' => $data->getPermissionNames(),
                ];
            }
            $collection = new Collection($table_data);
            return DataTables::of($collection)
                ->addIndexColumn()
                ->addColumn('action', function ($query) {
                    return view('admin.partials.action', [
                        'edit_route' => route('edit.role', ['role' => $query['id']]),
                        'delete_route' => route('delete.role', ['role' => $query['id']]),
                        'edit_permission' => 'edit-role',
                        'delete_permission' => 'delete-role',
                    ]);
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.roles.index');
    }

    public function create()
    {
        if (!$this->authorize('create-role')) {
            abort(403);
        }
        $newObj = new Role();
        if (auth()->user()->hasRole(User::SUPER_ADMIN_ROLE)) {
            $permissions = Permission::orderBy('name', 'ASC')->get();
        } else {
            $permissions = auth()->user()->getAllPermissions()->sortBy('name');
        }

        $permission_title = '';
        $permission_arr = array();
        foreach ($permissions as $permission) {
            $exploded = explode('-', $permission->name);

            if ($permission_title != $exploded[1]) {
                $permission_title = str_replace("_", " ", $exploded[1]);
            }

            $permission_arr[$permission_title][] = $permission;
        }

        $checked_roles = [];
        return view('admin.roles.form')
            ->with([
                'newObj' => $newObj,
                'permissions' => $permission_arr,
                'checked'  =>  $checked_roles,
            ]);
    }

    public function store(RoleStoreRequest $req)
    {
        // if (!$this->authorize('create-role')) {
        //     abort(403);
        // }
        // $input = $req->all();
        // $input['name'] = str_replace(' ', '-', strtolower($req->name));
        // $input['created_by'] = auth()->user()->id;
        // $input['updated_by'] = auth()->user()->id;
        // $role = Role::create($input);
        // if (!empty($input['permission'])) {
        //     $role->permissions()->sync($input['permission']);
        // }
        // toast("Data Saved Successfully!", 'success');
        return redirect()->route('roles');
    }

    public function edit(Role $role)
    {
        if (!$this->authorize('edit-role')) {
            abort(403);
        }
        if (auth()->user()->hasRole(User::SUPER_ADMIN_ROLE)) {
            $permissions = Permission::all();
        } else {
            $permissions = auth()->user()->getAllPermissions();
        }
        $permission_title = '';
        $permission_arr = array();
        foreach ($permissions as $permission) {
            $exploded = explode('-', $permission->name);
            if ($permission_title != $exploded[1]) {
                $permission_title = str_replace("_", " ", $exploded[1]);
            }
            $permission_arr[$permission_title][] = $permission;
        }
        $checked_roles = $role->getPermissionIdsAttribute();
        return view('admin.roles.edit')
            ->with([
                'newObj' => $role,
                'permissions' => $permission_arr,
                'checked'  =>  $checked_roles,
            ]);
    }

    public function update(Request $req, Role $role)
    {
        if (!$this->authorize('edit-role')) {
            abort(403);
        }
        $input = $req->all();
        $input['updated_by'] = auth()->user()->id;
        // $role->update($input);
        if (!empty($input['permission'])) {
            $role->permissions()->sync($input['permission']);
        } else {
            $role->permissions()->detach();
        }

        $output = Artisan::call('cache:clear');

        toast('Role updated', 'success');
        return redirect()->route('roles');
    }

    public function delete(Role $role)
    {
        if (!$this->authorize('delete-role')) {
            abort(403);
        }

        if ($role['name'] == User::FIELD_USER_ROLE || $role['name'] == User::SUPER_ADMIN_ROLE || $role['name'] == User::SUPERVISOR_ROLE || $role['name'] == User::DATA_ENTRY_ROLE) {
            toast('You can not delete system defined roles', 'error');
            return redirect()->back();
        }

        $role->delete();
        toast('Role Deleted Successfull!', 'success');
        return redirect()->back();
    }
}
