<?php

namespace App\Http\Controllers;

use App\Http\Requests\UserProfileUpdateRequest;
use App\Http\Requests\UserStoreRequest;
use App\Http\Requests\UserUpdateRequest;
use App\Jobs\UserConfirmationEmailQueueJob;
use App\Mail\UserAccountConfirmationEmail;
use App\Models\Asset;
use App\Models\City;
use App\Models\Country;
use App\Models\Region;
use App\Models\Role;
use App\Models\User;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Yajra\DataTables\DataTables;


class UserController extends Controller
{
    public function index(Request $req)
    {
        if (!$this->authorize('list-user')) {
            abort(403);
        }

        if ($req->ajax()) {
            $authUser = Auth::user();
            $roles = $authUser->getRoleNames();

            if ($roles->contains(User::SUPER_ADMIN_ROLE)) {
                $users = User::where('id', '!=', $authUser->id)->get();
            } elseif ($roles->contains(User::SUPERVISOR_ROLE)) {
                $users = User::where('id', '!=', $authUser->id)->where('city', $authUser->city)
                    ->whereDoesntHave('roles', function ($query) {
                        $query->where('name', User::SUPER_ADMIN_ROLE);
                    })
                    ->get();
            } else {
                $users = User::where('id', '!=', $authUser->id)
                    ->whereDoesntHave('roles', function ($query) {
                        $query->where('name', User::SUPER_ADMIN_ROLE);
                    })
                    ->get();
            }

            $table_data = [];

            foreach ($users as $user) {

                $task = 0; // Initialize $task variable

                $roles = $user->getRoleNames();
                $role_badges = [];

                if ($roles->contains(User::DATA_ENTRY_ROLE)) {
                    $task = Asset::where('asset_assign', $user->username)->get()->count();
                } elseif ($roles->contains(User::FIELD_USER_ROLE)) {
                    $task = Asset::where('asset_owner', $user->username)->get()->count();
                }

                // Create Bootstrap badges for each role
                foreach ($roles as $role) {
                    $role_badges[] = '<span class="badge badge-danger">' . $role . '</span>';
                }

                $table_data[] = [
                    'id' => $user->id,
                    'name' => $user->name,
                    'phone' => $user->phone,
                    'email' => $user->email,
                    'task' => $task,
                    'role' => implode(' ', $role_badges),

                ];
            }
            $collection = new Collection($table_data);
            return DataTables::of($collection)
                ->addIndexColumn()
                ->addColumn('action', function ($query) {
                    return view('admin.partials.action', [
                        'edit_route' => route('edit.user', ['user' => $query['id']]),
                        'delete_route' => route('delete.user', ['user' => $query['id']]),
                        'edit_permission' => 'edit-user',
                        'delete_permission' => 'delete-user',
                    ]);
                })
                ->rawColumns(['action', 'role'])
                ->make(true);
        }
        return view('admin.users.index');
    }

    public function edit(User $user)
    {
        if (!$this->authorize('edit-user')) {
            abort(403);
        }

        $task = 0; // Initialize $task variable
        $completeTask = 0; // Initialize $task variable

        $roles = $user->getRoleNames();

        if ($roles->contains(User::DATA_ENTRY_ROLE)) {
            $task = Asset::where('asset_assign', $user->username)->get()->count();
            $completeTask = Asset::where('user_status', 'Complete 🟢')->where('asset_assign', $user->username)->get()->count();
        } elseif ($roles->contains(User::FIELD_USER_ROLE)) {
            $task = Asset::where('asset_owner', $user->username)->get()->count();
        }

        $countries = Country::all();
        $regions = Region::all();
        $cities = City::all();
        $userRoles = $user->getRoleNames();
        $roles = Role::all();
        return view('admin.users.edit', ['user' => $user, 'roles' => $roles, 'userRoles' => $userRoles, 'countries' => $countries, 'regions' => $regions, 'cities' => $cities, 'task' => $task, 'completeTask' => $completeTask]);
    }

    public function update(Request $req, User $user)
    {
        if (!$this->authorize('edit-user')) {
            abort(403);
        }

        $req->validate([
            'name' => 'required|string|max:255',
            // 'username' => 'required|string|unique:users,username,' . $user->id,
            'email' => 'required|string|email|unique:users,email,' . $user->id,
            'phone' => 'nullable|string|unique:users,phone,' . $user->id,
        ]);

        $country = null;
        $region = null;
        $city = null;

        if (isset($req['country']) || isset($req['region']) || isset($req['city'])) {

            if (!empty($req['country'])) {
                $country = Country::findOrFail($req['country']);
            }
            if (!empty($req['region'])) {
                $region = Region::findOrFail($req['region']);
            }
            if (!empty($req['city'])) {
                $city = City::findOrFail($req['city']);
            }
        }

        $user->name = $req['name'];
        // $user->username = $req['username'];
        $user->email = $req['email'];
        $user->phone = $req['phone'];
        $user->country = $country->name ?? null;
        $user->region = $region->name ?? null;
        $user->city = $city->name ?? null;
        $user->building_number = $req['building_number'];
        $user->save();

        // Sync roles
        if ($req->has('role')) {
            $roles = $req['role'];
            $roleIds = Role::whereIn('name', $roles)->pluck('id')->toArray(); // Fetch role IDs based on role names
            $user->roles()->sync($roleIds); // Sync role IDs
        } else {
            // If no roles are selected, detach all roles
            $user->roles()->detach();
        }

        toast('User updated', 'success');
        return redirect()->route('users');
    }

    public function delete(User $user)
    {
        if (!$this->authorize('delete-user')) {
            abort(403);
        }
        $user->delete();
        toast('User Deleted!', 'success');
        return redirect()->back();
    }

    public function create()
    {
        if (!$this->authorize('create-user')) {
            abort(403);
        }
        $roles = Role::all();
        $countries = Country::all();
        $regions = Region::all();
        $cities = City::all();
        return view('admin.users.form', compact('roles', 'countries', 'regions', 'cities'));
    }

    public function store(UserStoreRequest $req)
    {
        if (!$this->authorize('create-user')) {
            abort(403);
        }

        $input = $req->all();
        $country = null;
        $region = null;
        $city = null;
        $phone = $input['country_code'] . $input['phone'];

        if (isset($input['country']) || isset($input['region']) || isset($input['city'])) {
            if (isset($input['country'])) {
                $country = Country::findOrFail($input['country']);
            }
            if (isset($input['region'])) {
                $region = Region::findOrFail($input['region']);
            }
            if (isset($input['city'])) {
                $city = City::findOrFail($input['city']);
            }
        }

        $existingUser = User::where('phone', $phone)->first();
        if ($existingUser) {
            return redirect()->back()->withErrors(['phone' => 'Phone number already exists.'])->withInput();
        }

        $user = User::create([
            'name' => $input['name'],
            'username' => $input['username'],
            'phone' => $phone,
            'email' => $input['email'],
            'password' => Hash::make($input['password']),
            'country' => $country ? $country->name : null,
            'region' => $region ? $region->name : null,
            'city' => $city ? $city->name : null,
            'building_number' => $input['building_number'],
            'remember_token'    => Str::random(60),
        ]);

        // Sync roles
        if ($req->has('role')) {
            $roles = $req['role'];
            $roleIds = Role::whereIn('name', $roles)->pluck('id')->toArray(); // Fetch role IDs based on role names
            $user->roles()->sync($roleIds); // Sync role IDs
        } else {
            // If no roles are selected, detach all roles
            $user->roles()->detach();
        }

        $mailData = [
            'title' => 'Mail From Qiam',
            'body' => "Hello $user->username! Your Account is Successfully Registered!",
            'password' => $input['password'],
            'email' => $input['email'],
        ];

        UserConfirmationEmailQueueJob::dispatch($mailData);

        toast('User Created Successfully!', 'success');
        return redirect()->route('users');
    }

    public function profile()
    {
        $roles = Auth::user()->getRoleNames();
        return view('admin.users.userProfile', compact('roles'));
    }

    public function profileUpdate(User $user, UserProfileUpdateRequest $req)
    {
        // Update basic profile information
        $user->name = $req['name'];
        $user->phone = $req['phone'];

        // Check if the current password is provided and matches the user's current password
        if ($req->filled('current_password')) {
            if (!Hash::check($req->current_password, $user->password)) {
                toast('Current password is incorrect.', 'error');
                return redirect()->back()->withErrors(['current_password' => 'The current password is incorrect.']);
            }
            // Update the password with the new one
            $user->password = Hash::make($req->new_password);
        }

        $user->save();

        toast('Profile updated', 'success');
        return redirect()->back();
    }

    public function getUsers(Request $req)
    {
        dd($req->all());
        $users = User::all();
        $asset = Asset::all();
        return response()->json(['users' => $users, 'asset' => $asset]);
    }
}
